@@ -13,12 +13,14 @@
 
 #include "DirTreeModel.h"
 #include "DirTree.h"
+#include "DirInfo.h"
 #include "FileInfoIterator.h"
 #include "DataColumns.h"
 #include "SelectionModel.h"
 #include "Settings.h"
 #include "SettingsHelpers.h"
 #include "Logger.h"
+#include "FormatUtil.h"
 #include "Exception.h"
 #include "DebugHelpers.h"
 
@@ -843,7 +845,7 @@ QString DirTreeModel::sizeText( FileInfo * item, QString (*fmtSz)(FileSize) )
 
 QString DirTreeModel::smallSizeText( FileInfo * item )
 {
-    if ( ! item->isFile() )
+    if ( ! item->isFile() && ! item->isSymLink() )
 	return "";
 
     FileSize allocated = item->allocatedSize();
@@ -878,13 +880,13 @@ QString DirTreeModel::smallSizeText( FileInfo * item )
 }
 
 
-bool DirTreeModel::isSmallFile( FileInfo * item )
+bool DirTreeModel::isSmallFileOrSymLink( FileInfo * item )
 {
-    if ( item			&&
-         item->isFile()		&&
-         item->blocks() > 0     &&
-         ! item->isSparseFile() &&
-         item->tree()		  )
+    if ( item			                  &&
+         ( item->isFile() || item->isSymLink()  ) &&
+         item->blocks() > 0                       &&
+         ! item->isSparseFile()                   &&
+         item->tree()		                    )
     {
         FileSize clusterSize = item->tree()->clusterSize();
 
@@ -915,16 +917,18 @@ QVariant DirTreeModel::sizeColText( FileInfo * item ) const
     if ( item->isDevice() )
 	return QVariant();
 
+    QString leftMargin( 2, ' ' );
+
     if ( item->isDirInfo() )
-	return item->sizePrefix() + formatSize( item->totalAllocatedSize() );
+	return leftMargin + item->sizePrefix() + formatSize( item->totalAllocatedSize() );
 
     QString text = sizeText( item );
 
-    if ( text.isEmpty() && isSmallFile( item ) )
+    if ( text.isEmpty() && isSmallFileOrSymLink( item ) )
 	text = smallSizeText( item );
 
     if ( text.isEmpty() )
-	text = formatSize( item->size() );
+	text = leftMargin + formatSize( item->size() );
 
     return text;
 }
@@ -1274,12 +1278,10 @@ void DirTreeModel::invalidatePersistent( FileInfo * subtree,
 
 QVariant DirTreeModel::formatPercent( float percent ) const
 {
-    if ( percent < 0.0 )	// Invalid percentage?
-	return QVariant();
+    QString text = ::formatPercent( percent );
 
-    QString text;
-    text.setNum( percent, 'f', 1 );
-    text += "%";
+    if ( text.isEmpty() )
+        return QVariant();
 
     return text;
 }
