@@ -8,6 +8,7 @@
 
 
 #include <QMenu>
+#include <QKeyEvent>
 
 #include "DirTreeView.h"
 #include "DirTreeModel.h"
@@ -18,6 +19,7 @@
 #include "SizeColDelegate.h"
 #include "HeaderTweaker.h"
 #include "DirTree.h"
+#include "FormatUtil.h"
 #include "Exception.h"
 #include "Logger.h"
 
@@ -28,7 +30,7 @@ DirTreeView::DirTreeView( QWidget * parent ):
     QTreeView( parent ),
     _cleanupCollection(0)
 {
-    _percentBarDelegate = new PercentBarDelegate( this );
+    _percentBarDelegate = new DirTreePercentBarDelegate( this, PercentBarCol );
     CHECK_NEW( _percentBarDelegate );
     setItemDelegateForColumn( PercentBarCol, _percentBarDelegate );
 
@@ -53,7 +55,6 @@ DirTreeView::DirTreeView( QWidget * parent ):
 
 DirTreeView::~DirTreeView()
 {
-    delete _percentBarDelegate;
     delete _headerTweaker;
 }
 
@@ -191,7 +192,7 @@ void DirTreeView::closeAllExcept( const QModelIndex & branch )
 	collapse( index );
     }
 
-    scrollTo( currentIndex() );
+    scrollTo( currentIndex(), QAbstractItemView::PositionAtCenter );
 }
 
 
@@ -211,3 +212,55 @@ void DirTreeView::setExpanded( FileInfo * item, bool expanded )
         QTreeView::setExpanded( index, expanded );
 }
 
+
+void DirTreeView::mousePressEvent( QMouseEvent * event )
+{
+    if ( event )
+    {
+        switch ( event->button() )
+        {
+            // Leave the the back / forward buttons on the mouse to act like the
+            // history back / forward buttons in the tool bar.
+            //
+            // By default, the QTreeView parent class uses them to act as
+            // cursor up / cursor down in the tree which defeats the idea of
+            // using them as history consistently throughout the application,
+            // making those mouse buttons pretty much unusable.
+            //
+            // So this makes sure those events are immediately propagated up to
+            // the parent widget.
+
+            case Qt::BackButton:
+            case Qt::ForwardButton:
+                event->ignore();
+                break;
+
+            default:
+                QTreeView::mousePressEvent( event );
+                break;
+        }
+    }
+}
+
+
+void DirTreeView::keyPressEvent( QKeyEvent * event )
+{
+    if ( event )
+    {
+        switch ( event->key() )
+        {
+            case Qt::Key_Asterisk:
+                // By default, this opens all tree branches which completely
+                // kills our performance, negating all our lazy sorting in
+                // each branch in the DirTreeModel / DirInfo classes.
+                //
+                // So let's just ignore this key; we have better alternatives
+                // with "Tree" -> "Expand to Level" -> "Level 0" .. "Level 5".
+                return;
+
+            default:
+                QTreeView::keyPressEvent( event );
+                break;
+        }
+    }
+}
