@@ -21,11 +21,9 @@
 
 namespace QDirStat
 {
-    class DirTree;
     class FileInfo;
     class FileTypeStats;
     class MimeCategory;
-    class SelectionModel;
     class LocateFileTypeWindow;
 
 
@@ -51,16 +49,13 @@ namespace QDirStat
 	 * of this class. The QPointer will keep track of this window
 	 * auto-deleting itself when closed.
 	 **/
-	FileTypeStatsWindow( SelectionModel * selectionModel,
-			     QWidget *	      parent );
+	FileTypeStatsWindow( QWidget * parent = 0 );
 
 	/**
 	 * Destructor.
 	 **/
 	virtual ~FileTypeStatsWindow();
 
-    public:
-
         /**
          * Obtain the subtree from the last used URL.
          **/
@@ -71,6 +66,24 @@ namespace QDirStat
 	 **/
 	void populate( FileInfo * subtree );
 
+        /**
+         * Convenience function for creating, populating and showing the shared
+         * instance.
+         **/
+        static void populateSharedInstance( FileInfo * subtree );
+
+        /**
+         * Static method for using one shared instance of this class between
+         * multiple parts of the application. This will create a new instance
+         * if there is none yet (or anymore).
+         *
+         * Do not hold on to this pointer; the instance destroys itself when
+         * the user closes the window, and then the pointer becomes invalid.
+         *
+         * After getting this shared instance, call populate() and show().
+         **/
+        static FileTypeStatsWindow * sharedInstance();
+
 
     public slots:
 
@@ -99,6 +112,7 @@ namespace QDirStat
 	 **/
 	virtual void reject() Q_DECL_OVERRIDE;
 
+
     protected slots:
 
 	/**
@@ -106,6 +120,7 @@ namespace QDirStat
 	 **/
 	void enableActions( QTreeWidgetItem * currentItem );
 
+
     protected:
 
 	/**
@@ -131,9 +146,10 @@ namespace QDirStat
 
 	Ui::FileTypeStatsWindow *   _ui;
         Subtree                     _subtree;
-	SelectionModel *	    _selectionModel;
 	FileTypeStats *		    _stats;
+
 	static QPointer<LocateFileTypeWindow> _locateFileTypeWindow;
+        static QPointer<FileTypeStatsWindow>  _sharedInstance;
     };
 
 
