@@ -17,38 +17,38 @@
 #include <QPointer>
 
 #include "ui_main-window.h"
-#include "FileTypeStatsWindow.h"
+#include "FileAgeStatsWindow.h"
 #include "FilesystemsWindow.h"
-#include "LocateFilesWindow.h"
-#include "TreeWalker.h"
+#include "HistoryButtons.h"
+#include "DiscoverActions.h"
 #include "PanelMessage.h"
-#include "UnreadableDirsWindow.h"
 #include "PkgFilter.h"
 #include "Subtree.h"
 
 
 class QCloseEvent;
-class QSortFilterProxyModel;
+class QMouseEvent;
 class QSignalMapper;
 class TreeLayout;
+class SysCallFailedException;
+class QMenu;
 
 
 namespace QDirStat
 {
-    class CleanupCollection;
     class ConfigDialog;
-    class DirTreeModel;
     class FileInfo;
-    class SelectionModel;
+    class DiscoverActions;
+    class PkgManager;
     class UnpkgSettings;
 }
 
+using QDirStat::FileAgeStatsWindow;
 using QDirStat::FileInfo;
-using QDirStat::FileTypeStatsWindow;
-using QDirStat::PanelMessage;
-using QDirStat::UnreadableDirsWindow;
 using QDirStat::FilesystemsWindow;
-using QDirStat::LocateFilesWindow;
+using QDirStat::PanelMessage;
+using QDirStat::PkgManager;
+using QDirStat::UnpkgSettings;
 
 
 class MainWindow: public QMainWindow
@@ -60,10 +60,6 @@ public:
     MainWindow();
     virtual ~MainWindow();
 
-    /**
-     * Return the DirTreeModel used in this window.
-     **/
-    QDirStat::DirTreeModel * dirTreeModel() const { return _dirTreeModel; }
 
 public slots:
 
@@ -203,22 +199,6 @@ public slots:
      **/
     void navigateToToplevel();
 
-    //
-    // "Discover" actions: Open a non-modal LocateFilesWindow that lists files
-    // of a certain category. A click on a file in that list opens the tree
-    // branch with that file in the main window and selects it.
-    //
-    // All of those actions re-use the same window.
-    //
-
-    void discoverLargestFiles();
-    void discoverNewestFiles();
-    void discoverOldestFiles();
-    void discoverHardLinkedFiles();
-    void discoverBrokenSymLinks();
-    void discoverSparseFiles();
-
-
     /**
      * Open the URL stored in an action's statusTip property with an external
      * browser.
@@ -310,20 +290,32 @@ protected slots:
      **/
     void cleanupFinished( int errorCount );
 
+    /**
+     * Navigate to the specified URL, i.e. make that directory the current and
+     * selected one; scroll there and open the tree branches so that URL is
+     * visible.
+     **/
+    void navigateToUrl( const QString & url );
+
     /**
      * Open the config dialog.
      **/
     void openConfigDialog();
 
+    /**
+     * Show file size statistics for the currently selected directory.
+     **/
+    void showFileSizeStats();
+
     /**
      * Show file type statistics for the currently selected directory.
      **/
     void showFileTypeStats();
 
     /**
-     * Show file size statistics for the currently selected directory.
+     * Show file age statistics for the currently selected directory.
      **/
-    void showFileSizeStats();
+    void showFileAgeStats();
 
     /**
      * Show detailed information about mounted filesystems in a separate window.
@@ -391,21 +383,27 @@ protected slots:
 
 protected:
 
-    /**
-     * Return the first selected directory or, if none is selected, the root
-     * directory.
-     **/
-    FileInfo * selectedDirOrRoot() const;
-
     /**
      * Set up QObject connections (all except from QActions)
      **/
     void connectSignals();
 
     /**
-     * Set up QObject connections from the actions from the .ui file
+     * Connect menu QActions from the .ui file to actions of this class
      **/
-    void connectActions();
+    void connectMenuActions();
+
+    void connectFileMenu();
+    void connectEditMenu();
+    void connectViewMenu();
+    void connectViewExpandMenu();
+    void connectViewTreemapMenu();
+    void connectGoMenu();
+    void connectDiscoverMenu();
+    void connectHelpMenu();
+    void connectHelpSolutionsMenu();
+
+    void connectDebugActions();
 
     /**
      * Set up the _treeLevelMapper to map an "expand tree to level x" action to
@@ -413,16 +411,6 @@ protected:
      **/
     void mapTreeExpandAction( QAction * action, int level );
 
-    /**
-     * Common part of all "discover" actions: Create or reuse a
-     * LocateFilesWindow with the specified TreeWalker.
-     *
-     * 'headingText' should include a "%1" placeholder for the path of the
-     * starting directory.
-     **/
-    void discoverFiles( QDirStat::TreeWalker * treeWalker,
-                        const QString &        headingText );
-
     /**
      * Initialize the layout actions.
      **/
@@ -454,32 +442,58 @@ protected:
     void writeLayoutSettings( TreeLayout * layout );
 
     /**
-     * Window close event ([x] icon in the top right window)
+     * Check for package manager support and enable or disable some of the
+     * related actions in the menus accordingly.
      **/
-    virtual void closeEvent( QCloseEvent *event ) Q_DECL_OVERRIDE;
+    void checkPkgManagerSupport();
 
     /**
-     * Format a millisecond-based time
+     * Apply the exclude rules from 'unpkgSettings' to the DirTree.
      **/
-    QString formatTime( qint64 millisec, bool showMillisec = true );
+    void setUnpkgExcludeRules( const UnpkgSettings & unpkgSettings );
+
+    /**
+     * Apply the filters to the DirTree:
+     * - Ignore all files that belong to an installed package
+     * - Ignore all file patterns ("*.pyc" etc.) the user wishes to ignore
+     **/
+    void setUnpkgFilters( const UnpkgSettings & unpkgSettings,
+                          PkgManager          * pkgManager );
+
+    /**
+     * Parse the starting directory in the 'unpkgSettings' and remove the
+     * starting "unpkg:" part to it is suitable for actually opening a
+     * directory. Return the parsed directory path.
+     **/
+    QString parseUnpkgStartingDir( const UnpkgSettings & unpkgSettings );
+
+    /**
+     * Show an error popup that a directory could not be opened and wait until
+     * the user confirmed it.
+     *
+     * The relevant informatoin is all in the exception.
+     **/
+    void showOpenDirErrorPopup( const SysCallFailedException & ex );
+
+    /**
+     * Handle mouse buttons: Activate history actions actionGoBack and
+     * actionGoForward with the "back" and "forward" mouse buttons as well.
+     **/
+    virtual void mousePressEvent( QMouseEvent * event ) Q_DECL_OVERRIDE;
 
 
 private:
 
-    Ui::MainWindow		*  _ui;
-    QDirStat::DirTreeModel	*  _dirTreeModel;
-    QDirStat::SelectionModel	*  _selectionModel;
-    QDirStat::CleanupCollection *  _cleanupCollection;
-    QDirStat::ConfigDialog	*  _configDialog;
-    QActionGroup		*  _layoutActionGroup;
-    QPointer<FileTypeStatsWindow>  _fileTypeStatsWindow;
+    Ui::MainWindow		 * _ui;
+    QDirStat::ConfigDialog	 * _configDialog;
+    QDirStat::HistoryButtons     * _historyButtons;
+    QDirStat::DiscoverActions    * _discoverActions;
+    QActionGroup		 * _layoutActionGroup;
+    QPointer<FileAgeStatsWindow>   _fileAgeStatsWindow;
     QPointer<FilesystemsWindow>    _filesystemsWindow;
-    QPointer<LocateFilesWindow>    _locateFilesWindow;
     QPointer<PanelMessage>	   _dirPermissionsWarning;
-    QPointer<UnreadableDirsWindow> _unreadableDirsWindow;
     QString			   _dUrl;
     QElapsedTimer		   _stopWatch;
-    bool			   _modified;
     bool			   _enableDirPermissionsWarning;
     bool			   _verboseSelection;
     bool			   _urlInWindowTitle;
@@ -498,6 +512,9 @@ private:
 
 /**
  * Helper class for the different layouts of the tree view layout.
+ *
+ * Notice that the column layouts are handled in the HeaderTweaker and its
+ * ColumnLayout helper class; see also HeaderTweaker.h and HeaderTweaker.cpp.
  **/
 class TreeLayout
 {
