@@ -18,6 +18,7 @@
 
 #if (QT_VERSION < QT_VERSION_CHECK( 5, 4, 0 ))
 #  define HAVE_Q_STORAGE_INFO 0
+typedef void * QStorageInfo;
 #else
 #  define HAVE_Q_STORAGE_INFO 1
 #  include <QStorageInfo>
@@ -42,8 +43,13 @@ namespace QDirStat
 		    const QString & filesystemType,
 		    const QString & mountOptions );
 
+        /**
+         * Destructor.
+         **/
+        virtual ~MountPoint();
+
 	/**
-	 * Return the device that is mounted, someting like "/dev/sda3",
+	 * Return the device that is mounted, something like "/dev/sda3",
 	 * "/dev/mapper/crypto", "nas:/share/work".
 	 **/
 	QString device() const { return _device; }
@@ -107,7 +113,7 @@ namespace QDirStat
         /**
          * Return 'true' if this is an autofs that is not currently mounted.
          **/
-        bool isUnmountedAutofs() const;
+        bool isUnmountedAutofs();
 
 	/**
 	 * Return 'true' if this is a duplicate mount, i.e. either a bind mount
@@ -137,34 +143,42 @@ namespace QDirStat
 	 * Total size of the filesystem of this mount point.
 	 * This returns -1 if no size information is available.
 	 **/
-	FileSize totalSize() const;
+	FileSize totalSize();
 
 	/**
 	 * Total used size of the filesystem of this mount point.
 	 * This returns -1 if no size information is available.
 	 **/
-	FileSize usedSize() const;
+	FileSize usedSize();
 
 	/**
 	 * Reserved size for root for the filesystem of this mount point.
 	 * This returns -1 if no size information is available.
 	 **/
-	FileSize reservedSize() const;
+	FileSize reservedSize();
 
 	/**
 	 * Available free size of this filesystem for non-privileged users.
 	 * This returns -1 if no size information is available.
 	 **/
-	FileSize freeSizeForUser() const;
+	FileSize freeSizeForUser();
 
 	/**
 	 * Available free size of this filesystem for privileged users.
 	 * This returns -1 if no size information is available.
 	 **/
-	FileSize freeSizeForRoot() const;
+	FileSize freeSizeForRoot();
 
 
     protected:
+
+#if HAVE_Q_STORAGE_INFO
+        /**
+         * Lazy access to the QStorageInfo for this mount.
+         **/
+        QStorageInfo * storageInfo();
+#endif
+
 	QString	    _device;
 	QString	    _path;
 	QString	    _filesystemType;
@@ -172,7 +186,7 @@ namespace QDirStat
 	bool	    _isDuplicate;
 
 #if HAVE_Q_STORAGE_INFO
-	QStorageInfo _storageInfo;
+	QStorageInfo * _storageInfo;
 #endif
     }; // class MountPoint
 
@@ -294,6 +308,26 @@ namespace QDirStat
 	 **/
 	bool read( const QString & filename );
 
+#if HAVE_Q_STORAGE_INFO
+
+        /**
+         * Fallback method if neither /proc/mounts nor /etc/mtab is available:
+         * Try using QStorageInfo. Return 'true' if any mount point was found.
+         **/
+        bool readStorageInfo();
+#endif
+
+        /**
+         * Post-process a mount point and add it to the internal list and map.
+         **/
+        void postProcess( MountPoint * mountPoint );
+
+        /**
+         * Add a mount point to the internal list and map.
+         **/
+        void add( MountPoint * mountPoint );
+
+
 	/**
 	 * Check if any of the mount points has filesystem type "btrfs".
 	 **/
@@ -301,9 +335,9 @@ namespace QDirStat
 
         /**
          * Try to check with the external "lsblk" command (if available) what
-         * block devices use NTFS and return those device names in a list.
+         * block devices use NTFS and populate _ntfsDevices with them.
          **/
-        QStringList findNtfsDevices();
+        void findNtfsDevices();
 
 	/**
 	 * Return 'true' if 'device' is mounted.
@@ -318,6 +352,7 @@ namespace QDirStat
 
 	QList<MountPoint *>	    _mountPointList;
 	QMap<QString, MountPoint *> _mountPointMap;
+        QStringList                 _ntfsDevices;
 	bool			    _isPopulated;
 	bool			    _hasBtrfs;
 	bool			    _checkedForBtrfs;
