@@ -104,7 +104,9 @@ void TreemapTile::init()
 
     setFlags( ItemIsSelectable );
     _highlighter = 0;
-    setAcceptHoverEvents(true);
+
+    if ( ( _orig->isDir() && _orig->totalSubDirs() == 0 ) || _orig->isDotEntry() )
+        setAcceptHoverEvents( true );
 
     if ( ! _parentTile )
 	_parentView->scene()->addItem( this );
@@ -497,14 +499,14 @@ QPixmap TreemapTile::renderCushion()
 	}
     }
 
-    if ( _parentView->ensureContrast() )
-	ensureContrast( image );
+    if ( _parentView->enforceContrast() )
+	enforceContrast( image );
 
     return QPixmap::fromImage( image );
 }
 
 
-void TreemapTile::ensureContrast( QImage & image )
+void TreemapTile::enforceContrast( QImage & image )
 {
     if ( image.width() > 5 )
     {
@@ -630,6 +632,23 @@ void TreemapTile::mousePressEvent( QGraphicsSceneMouseEvent * event )
 	    _parentView->setCurrentItem( this );
 	    break;
 
+	case Qt::MidButton:
+            // logDebug() << "Middle click on " << _orig << endl;
+
+            // Handle item selection (with or without Ctrl) ourselves here;
+            // unlike for a left click, the QGraphicsItem base class does
+            // not do this for us already.
+
+            if ( ( event->modifiers() & Qt::ControlModifier ) == 0 )
+                scene()->clearSelection();
+
+            setSelected( ! isSelected() );
+
+            _parentView->setCurrentItem( this );
+            _parentView->toggleParentsHighlight( this );
+            break;
+
+
 	case Qt::RightButton:
 	    // logDebug() << this << " right mouse pressed" << endl;
 	    _parentView->setCurrentItem( this );
@@ -658,27 +677,6 @@ void TreemapTile::mouseReleaseEvent( QGraphicsSceneMouseEvent * event )
 	    }
 	    break;
 
-	case Qt::MidButton:
-	    {
-		logDebug() << "Selecting parent" << endl;
-
-		TreemapTile * oldCurrentTile = _parentView->currentItem();
-		TreemapTile * newCurrentTile = this;
-
-		// Select the next-higher ancestor if possible
-		if ( oldCurrentTile &&
-		     oldCurrentTile->parentTile() &&
-		     _orig->isInSubtree( oldCurrentTile->parentTile()->orig() ) )
-		{
-		    newCurrentTile = oldCurrentTile->parentTile();
-		}
-
-		_parentView->currentItem()->setSelected( false );
-		newCurrentTile->setSelected( true );
-		_parentView->setCurrentItem( newCurrentTile );
-	    }
-	    break;
-
 	default:
 	    QGraphicsRectItem::mouseReleaseEvent( event );
 	    break;
