@@ -29,6 +29,14 @@
 using namespace QDirStat;
 
 
+// Treemap layers (Z values)
+
+const double TileLayer           = 0.0;
+const double SceneMaskLayer      = 1e5;
+const double TileHighlightLayer  = 1e6;
+const double SceneHighlightLayer = 1e10;
+
+
 TreemapView::TreemapView( QWidget * parent ):
     QGraphicsView( parent ),
     _tree(0),
@@ -39,7 +47,9 @@ TreemapView::TreemapView( QWidget * parent ):
     _rootTile(0),
     _currentItem(0),
     _currentItemRect(0),
+    _sceneMask(0),
     _newRoot(0),
+    _highlightedTile(0),
     _useFixedColor(false),
     _useDirGradient(true)
 {
@@ -82,6 +92,8 @@ void TreemapView::clear()
     _currentItem     = 0;
     _currentItemRect = 0;
     _rootTile	     = 0;
+    _sceneMask       = 0;
+    _parentHighlightList.clear();
 }
 
 
@@ -160,7 +172,7 @@ void TreemapView::readSettings()
     _heightScaleFactor	= settings.value( "HeightScaleFactor", DefaultHeightScaleFactor ).toDouble();
     _squarify		= settings.value( "Squarify"	     , true  ).toBool();
     _doCushionShading	= settings.value( "CushionShading"   , true  ).toBool();
-    _ensureContrast	= settings.value( "EnsureContrast"   , true  ).toBool();
+    _enforceContrast	= settings.value( "EnforceContrast"  , false ).toBool();
     _forceCushionGrid	= settings.value( "ForceCushionGrid" , false ).toBool();
     _useDirGradient	= settings.value( "UseDirGradient"   , true  ).toBool();
     _minTileSize	= settings.value( "MinTileSize"	     , DefaultMinTileSize ).toInt();
@@ -189,7 +201,7 @@ void TreemapView::writeSettings()
     settings.setValue( "HeightScaleFactor" , _heightScaleFactor	 );
     settings.setValue( "Squarify"	   , _squarify		 );
     settings.setValue( "CushionShading"	   , _doCushionShading	 );
-    settings.setValue( "EnsureContrast"	   , _ensureContrast	 );
+    settings.setValue( "EnforceContrast"   , _enforceContrast	 );
     settings.setValue( "ForceCushionGrid"  , _forceCushionGrid	 );
     settings.setValue( "UseDirGradient"	   , _useDirGradient	 );
     settings.setValue( "MinTileSize"	   , _minTileSize	 );
@@ -318,11 +330,9 @@ void TreemapView::rebuildTreemap( FileInfo *	 newRoot,
     if ( newSz.isEmpty() )
 	newSize = visibleSize();
 
-
     // Delete all old stuff.
     clear();
 
-
     if ( ! scene() )
     {
 	QGraphicsScene * scene = new QGraphicsScene( this );
@@ -488,6 +498,9 @@ void TreemapView::setCurrentItem( TreemapTile * tile )
 
     if ( _currentItem )
     {
+        if ( highlightedParent() != _currentItem->parentTile() )
+            clearParentsHighlight();
+
 	if ( ! _currentItemRect )
 	    _currentItemRect = new CurrentItemHighlighter( scene(), _currentItemColor );
     }
@@ -670,6 +683,82 @@ QColor TreemapView::tileColor( FileInfo * file )
 }
 
 
+void TreemapView::highlightParents( TreemapTile * tile )
+{
+    if ( ! tile )
+    {
+        clearParentsHighlight();
+        return;
+    }
+
+    _highlightedTile = tile;
+    TreemapTile * parent = tile->parentTile();
+    TreemapTile * currentHighlight = highlightedParent();
+
+    if ( currentHighlight && currentHighlight != parent )
+        clearParentsHighlight();
+
+    TreemapTile * topParent = 0;
+    int lineWidth = 2;  // For the first (the direct) parent
+
+    while ( parent && parent != _rootTile )
+    {
+        HighlightRect * highlight = new HighlightRect( parent, Qt::white, lineWidth );
+        CHECK_NEW( highlight );
+        _parentHighlightList << highlight;
+        highlight->setToolTip( parent->orig()->debugUrl() );
+
+        topParent = parent;
+        parent = parent->parentTile();
+        lineWidth = 1;  // For all higher-level parents
+    }
+
+    if ( topParent )
+    {
+        clearSceneMask();
+        _sceneMask = new SceneMask( topParent, 0.6 );
+    }
+}
+
+
+void TreemapView::clearParentsHighlight()
+{
+    qDeleteAll( _parentHighlightList );
+    _parentHighlightList.clear();
+    _highlightedTile = 0;
+    clearSceneMask();
+}
+
+
+void TreemapView::toggleParentsHighlight( TreemapTile * tile )
+{
+    if ( tile == _highlightedTile )
+        clearParentsHighlight();
+    else
+        highlightParents( tile );
+}
+
+
+void TreemapView::clearSceneMask()
+{
+    if ( _sceneMask )
+        delete _sceneMask;
+
+    _sceneMask = 0;
+}
+
+
+TreemapTile * TreemapView::highlightedParent() const
+{
+    TreemapTile * tile = 0;
+
+    if ( ! _parentHighlightList.empty() )
+        tile = _parentHighlightList.first()->tile();
+
+    return tile;
+}
+
+
 void TreemapView::sendHoverEnter( FileInfo * node )
 {
     emit hoverEnter( node );
@@ -687,30 +776,54 @@ void TreemapView::sendHoverLeave( FileInfo * node )
 
 
 HighlightRect::HighlightRect( QGraphicsScene * scene, const QColor & color, int lineWidth ):
-    QGraphicsRectItem()
+    QGraphicsRectItem(),
+    _tile(0)
 {
     QPen pen( color, lineWidth );
     pen.setStyle( Qt::DotLine );
     setPen( QPen( color, lineWidth ) );
     setPen( pen );
-    setZValue( 1e10 );		// Higher than everything else
+    setZValue( SceneHighlightLayer );
     hide();
     scene->addItem( this );
 }
 
 
 HighlightRect::HighlightRect( TreemapTile * tile, const QColor & color, int lineWidth ):
-    QGraphicsRectItem()
+    QGraphicsRectItem(),
+    _tile( tile )
 {
     CHECK_PTR( tile );
 
     setPen( QPen( color, lineWidth ) );
-    setZValue( 1e8 );	       // Not quite as high as the scene-wide highlight rect
+    setZValue( TileHighlightLayer + tile->zValue() );
     tile->scene()->addItem( this );
     highlight( tile );
 }
 
 
+QPainterPath HighlightRect::shape() const
+{
+    if ( ! _tile )
+        return QGraphicsRectItem::shape();
+
+    // Return just the outline as the shape so any tooltip is only displayed on
+    // the outline, not inside as well; but use more than the line thickness of
+    // 1 or 2 pixels to make it humanly possible to position the mouse cursor
+    // close enough.
+    //
+    // Notice that it's still only on the inside of the line to avoid bad side
+    // effects with QGraphicsView's internal mechanisms.
+
+    const int thickness = 10;
+
+    QPainterPath path;
+    path.addRect( _tile->rect() );
+    path.addRect( _tile->rect().adjusted( thickness,   thickness,
+                                          -thickness, -thickness ) );
+    return path;
+}
+
 
 void HighlightRect::highlight( TreemapTile * tile )
 {
@@ -755,3 +868,30 @@ void CurrentItemHighlighter::highlight( TreemapTile * tile )
     setPenStyle( tile );
 }
 
+
+
+
+SceneMask::SceneMask( TreemapTile * tile, float opacity ):
+    QGraphicsPathItem(),
+    _tile( tile )
+{
+    // logDebug() << "Adding scene mask for " << tile->orig() << endl;
+    CHECK_PTR( tile );
+
+    QPainterPath path;
+    path.addRect( tile->scene()->sceneRect() );
+
+    // Since the default OddEvenFillRule leaves overlapping areas unfilled,
+    // adding the tile's rect that is inside the scene rect leaves the tile
+    // "cut out", i.e. unobscured.
+
+    path.addRect( tile->rect() );
+    setPath( path );
+
+    const int grey = 0x30;
+    QColor color( grey, grey, grey, opacity * 255 );
+    setBrush( color );
+
+    setZValue( SceneMaskLayer + tile->zValue() );
+    tile->scene()->addItem( this );
+}
