@@ -13,6 +13,8 @@
 
 #include <QGraphicsView>
 #include <QGraphicsRectItem>
+#include <QGraphicsPathItem>
+#include <QList>
 
 #include "FileInfo.h"
 
@@ -38,6 +40,7 @@ namespace QDirStat
 {
     class TreemapTile;
     class HighlightRect;
+    class SceneMask;
     class DirTree;
     class SelectionModel;
     class SelectionModelProxy;
@@ -45,6 +48,8 @@ namespace QDirStat
     class FileInfoSet;
     class DelayedRebuilder;
 
+    typedef QList<HighlightRect *> HighlightRectList;
+
 
     /**
      * View widget that displays a DirTree as a treemap.
@@ -88,6 +93,12 @@ namespace QDirStat
 	 **/
 	TreemapTile * rootTile() const { return _rootTile; }
 
+        /**
+         * Returns the currently highlighted treemap tile (that was highlighted
+         * with a middle click) or 0 if there is none.
+         **/
+        TreemapTile * highlightedTile() const { return _highlightedTile; }
+
 	/**
 	 * Returns this treemap view's DirTree.
 	 **/
@@ -137,6 +148,13 @@ namespace QDirStat
 	 **/
 	void setFixedColor( const QColor & fixedColor );
 
+        /**
+         * Return the tile of the deepest-level highlighted parent or 0 if no
+         * parent is currently highlighted. Notice that this returns the real
+         * tile corresponding to a directory, not the HighlightRect.
+         **/
+        TreemapTile * highlightedParent() const;
+
 
     public slots:
 
@@ -227,6 +245,27 @@ namespace QDirStat
          **/
         void sendHoverLeave( FileInfo * node );
 
+        /**
+         * Highlight the parent tiles of item 'tile'.
+         **/
+        void highlightParents( TreemapTile * tile );
+
+        /**
+         * Clear previous parent highlights.
+         **/
+        void clearParentsHighlight();
+
+        /**
+         * Highlight the parent tiles of item 'tile' if that tile is not
+         * currently highlighted, or clear the highlight if it is.
+         **/
+        void toggleParentsHighlight( TreemapTile * tile );
+
+        /**
+         * Clear the old scene mask if there is one.
+         **/
+        void clearSceneMask();
+
 	/**
 	 * Read parameters from the settings file.
 	 **/
@@ -262,7 +301,7 @@ namespace QDirStat
 
 	/**
 	 * Schedule a rebuild of the treemap with 'newRoot'. If another rebuild
-	 * is scheduled before the timout is over, nothing will happen until
+	 * is scheduled before the timeout is over, nothing will happen until
 	 * the last scheduled timeout has elapsed.
 	 *
 	 * The purpose of this is to avoid unnecessary rebuilds when the user
@@ -337,7 +376,7 @@ namespace QDirStat
 	 * Returns 'true' if tile boundary lines should be drawn for cushion
 	 * treemaps, 'false'  if not.
 	 **/
-	bool ensureContrast() const { return _ensureContrast; }
+	bool enforceContrast() const { return _enforceContrast; }
 
 	/**
 	 * Returns the minimum tile size in pixels. No treemap tiles less than
@@ -457,7 +496,7 @@ namespace QDirStat
     protected slots:
 
 	/**
-	 * Rebuild the treemap if no more pendung rebuilds are scheduled.
+	 * Rebuild the treemap if no more pending rebuilds are scheduled.
 	 **/
 	void rebuildTreemapDelayed();
 
@@ -483,13 +522,16 @@ namespace QDirStat
 	TreemapTile	    * _rootTile;
 	TreemapTile	    * _currentItem;
 	HighlightRect	    * _currentItemRect;
+        SceneMask           * _sceneMask;
 	FileInfo	    * _newRoot;
+        TreemapTile         * _highlightedTile;
+        HighlightRectList     _parentHighlightList;
 	QString		      _savedRootUrl;
 
 	bool   _squarify;
 	bool   _doCushionShading;
 	bool   _forceCushionGrid;
-	bool   _ensureContrast;
+	bool   _enforceContrast;
 	bool   _useFixedColor;
 	int    _minTileSize;
         bool   _useDirGradient;
@@ -564,6 +606,24 @@ namespace QDirStat
 	 **/
 	void setPenStyle( TreemapTile * tile );
 
+        /**
+         * Return the tile that this highlights or 0 if there is none.
+         **/
+        TreemapTile * tile() const { return _tile; }
+
+        /**
+         * Return the shape of this item; in this case only the outline,
+         * leaving the inside hollow to avoid displaying the tooltip there as
+         * well.
+         *
+         * Reimplemented from QGraphicsRectItem / QGraphicsItem.
+         **/
+        virtual QPainterPath shape() const Q_DECL_OVERRIDE;
+
+    protected:
+
+        TreemapTile * _tile;
+
     }; // class TreemapSelectionRect
 
 
@@ -598,6 +658,34 @@ namespace QDirStat
 	    {}
     };
 
+
+
+    /**
+     * Semi-transparent mask that covers the complete scene except for one
+     * tile.
+     **/
+    class SceneMask: public QGraphicsPathItem
+    {
+    public:
+        /**
+         * Constructor: Create a semi-transparent mask that covers the complete
+         * scene (the complete treemap), but leaves 'tile' uncovered.
+         *
+         * 'opacity' (0.0 .. 1.0) indicates how transparent the mask is:
+         * 0.0 -> completely transparent; 1.0 -> solid.
+         **/
+        SceneMask( TreemapTile * tile, float opacity );
+
+        /**
+         * Return the tile that this masks.
+         **/
+        TreemapTile * tile() const { return _tile; }
+
+    protected:
+
+        TreemapTile * _tile;
+    };
+
 }	// namespace QDirStat
 
 
